'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _ttyTable = require('tty-table');

var _ttyTable2 = _interopRequireDefault(_ttyTable);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _constant = require('../common/constant');

var _core = require('../common/core');

var _common = require('../common');

var _lifecycle = require('./lifecycle');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (repositories, command) {
  if (!_crossSpawn2.default.sync('git', 'config --get remote.origin.url'.split(' ')).stdout.toString()) {
    console.log('\u975EGit\u5DE5\u7A0B\u76EE\u5F55\uFF0C\u4E0D\u80FD\u6267\u884C\u5408\u5305\u64CD\u4F5C'.red);
    process.exit();
  }
  var startTime = Date.now();
  var _command$env = command.env,
      env = _command$env === undefined ? 'dev' : _command$env,
      _command$debug = command.debug,
      debug = _command$debug === undefined ? false : _command$debug;

  (0, _lifecycle.beforeCommand)();

  var _getWorkDirs = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs.DIST_DIR;

  var isAllSucceed = true;

  _fsExtra2.default.removeSync(DIST_DIR);
  _fs2.default.mkdirSync(DIST_DIR);
  if (!_fs2.default.existsSync('./project.config.json')) {
    throw new ReferenceError('找不到小程序项目配置文件: project.config.json，无法合包');
  }

  (0, _common.executeAssemble)(repositories, { debug: debug, env: env }).then(function (report) {
    console.log('报告信息', report);
    console.log(_constant.EOL);
    var header = [{
      value: '包名',
      width: 20
    }, {
      value: '仓库地址',
      width: 60
    }, {
      value: '分支',
      width: 20
    }, {
      value: 'SHA1',
      width: 60
    }, {
      value: '作者',
      width: 20
    }, {
      value: '合包结果',
      width: 20
    }, {
      value: '失败原因',
      width: 60
    }];
    var rows = [];
    Object.keys(report).forEach(function (name) {
      var _report$name = report[name],
          url = _report$name.url,
          commit = _report$name.commit,
          author = _report$name.author,
          branch = _report$name.branch;

      var _parseGitUrl = (0, _core.parseGitUrl)(url),
          repository = _parseGitUrl.repository;

      var isPackageFail = report[name].isFail;
      var color = isPackageFail ? 'red' : 'green';
      isPackageFail && (isAllSucceed = false);
      rows.push([name[color], repository[color], branch[color], commit[color], author[color], (report[name].isFail ? '失败' : '成功')[color], report[name].reasons.join('\n')[color]]);
    });
    console.log((0, _ttyTable2.default)(header, rows, {
      align: 'left'
    }).render());
    console.log(_constant.EOL);

    var logName = (0, _core.exportAssembleReport)(report);

    if (_os2.default.type().toLowerCase() === 'darwin') {
      (0, _common.openWeChatIDE)();
    }
    console.log('合并代码所在目录：\n'.green);
    console.log(DIST_DIR.yellow);
    console.log('\u672C\u6B21\u5408\u5305\u8017\u65F6: ', String((Date.now() - startTime) / 1000).yellow, 's');

    // 异步：prod 环境下对相应分支打 Tag
    Promise.resolve(true).then(function () {
      if (env === 'prod') {
        return _inquirer2.default.prompt([{
          name: 'confirm',
          type: 'confirm',
          message: '是否需要打Tag'
        }]).then(function (answers) {
          if (!answers.confirm) {
            return process.exit(0);
          }

          return [{
            name: 'tag',
            message: '请输入Tag: '
          }, {
            name: 'message',
            message: '请输入Tag message: '
          }];
        }).then(_inquirer2.default.prompt).then(function (answers) {
          return (0, _core.buildTag)(answers, logName);
        }).catch(function (e) {
          console.log(e);
        });
      }

      return true;
    }).then(function () {
      (0, _lifecycle.afterCommand)();
    });
  }).catch(function (e) {
    console.log(e);
    return e;
  });
};