'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _gulp = require('gulp');

var _gulp2 = _interopRequireDefault(_gulp);

var _gulpFilter = require('gulp-filter');

var _gulpFilter2 = _interopRequireDefault(_gulpFilter);

var _gulpChange = require('gulp-change');

var _gulpChange2 = _interopRequireDefault(_gulpChange);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _constant = require('../common/constant');

var _core = require('../common/core');

var _common = require('../common');

var _assemble = require('./assemble');

var _assemble2 = _interopRequireDefault(_assemble);

var _lifecycle = require('./lifecycle');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var REQUIRED_FILES = ['components/**', 'constant/**', 'images/**', 'pages/**', 'third/**', 'app.wxss', 'app.js'];
var SPECIAL_FILES = ['main-app.js', 'sub-app.js', 'app.json'];

var PROJECT_CONFIG_FILE = 'project.config.json';

function transfer(src, dest, callback) {
  var _getWorkDirs = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs.DIST_DIR;

  var f = (0, _gulpFilter2.default)(['sub-app.js', 'main-app.js', 'app.json'], { restore: true });
  _gulp2.default.src(src, { base: '.' }).pipe(f).pipe((0, _gulpChange2.default)(function (content) {
    var file = this.file;
    var basename = _path2.default.basename(file.path);
    var filePath = _path2.default.relative(file.cwd, file.path);

    console.log(('\u590D\u5236\u6587\u4EF6' + filePath + ' >> ' + dest + _constant.SEP + filePath).green);

    if (SPECIAL_FILES.indexOf(basename) > -1) {
      console.log(('\u5904\u7406\u7279\u522B\u6587\u4EF6' + filePath + '...').green);
      var isMain = (0, _core.isMainPackage)();
      if (isMain) {
        if (basename === 'app.json') {
          var appJson = JSON.parse(content);
          var distAppJsonPath = '' + DIST_DIR + _constant.SEP + filePath;
          var distAppJson = _fs2.default.existsSync(distAppJsonPath) && _fsExtra2.default.readJsonSync('' + DIST_DIR + _constant.SEP + filePath);
          distAppJson = distAppJson && Object.assign(distAppJson, appJson) || appJson;
          content = JSON.stringify(distAppJson, null, _constant.INDENT);
        } else if (basename === 'main-app.js') {
          var weshopConfig = _fsExtra2.default.readJsonSync(_path2.default.join(DIST_DIR, 'weshop.config.json'));
          var mainAppOptions = (0, _core.getAppOptions)(file.path);
          weshopConfig && weshopConfig.subPackages && weshopConfig.subPackages.forEach(function (subPackage) {
            var subPackageName = (0, _core.getProjectNameFromGitUrl)(subPackage);
            if (_fs2.default.existsSync(_path2.default.join(DIST_DIR, subPackageName))) {
              var subAppOptions = (0, _core.getAppOptions)(_path2.default.join(DIST_DIR, subPackageName, 'sub-app.js'));
              Object.assign(mainAppOptions.globalData, subAppOptions.globalData);
              content = (0, _core.wrapAppJsOptions)(mainAppOptions);
            }
          });
        }
      } else {
        (0, _common.executeAssemble)();
      }
    }
    return content;
  })).pipe(f.restore).pipe(_gulp2.default.dest(dest)).on('end', function () {
    callback && callback();
  });
}

exports.default = function () {
  (0, _lifecycle.beforeCommand)();
  console.log('正在初始化开发模式...'.green);

  var _getWorkDirs2 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs2.DIST_DIR;

  var isMain = (0, _core.isMainPackage)();
  var packageName = (0, _core.getProjectNameFromGitUrl)();
  var destPath = '' + DIST_DIR + (isMain ? '' : _constant.SEP + packageName);

  if (!isMain) {
    (0, _assemble2.default)();
  } else {
    transfer(REQUIRED_FILES.concat(SPECIAL_FILES).concat([PROJECT_CONFIG_FILE]), destPath, _common.openWeChatIDE);
  }

  _gulp2.default.watch(REQUIRED_FILES.concat(SPECIAL_FILES), { base: '.' }, function (event) {
    transfer(event.path, destPath);
  });

  console.log('开发模式初始化完成，开始实时编译...'.green);

  (0, _lifecycle.afterCommand)();
};