'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _common = require('../common');

var _scaffold = require('../common/scaffold');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _constant = require('../common/constant');

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _lifecycle = require('./lifecycle');

var _core = require('../common/core');

var _update = require('./update');

var _update2 = _interopRequireDefault(_update);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function () {
  var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

  var absolutePath = _path2.default.join(process.cwd(), name);
  _fsExtra2.default.ensureDirSync(absolutePath);
  process.chdir(absolutePath);

  if (!(0, _core.isEmptyDirectory)(process.cwd(), ['.git'])) {
    console.log(absolutePath.blue, '非空目录，请清空目录或另找一个空目录执行'.red);
    process.exit();
  }

  // 设置传入的 name
  _constant.QUESTIONS[0].default = name;

  _inquirer2.default.prompt(_constant.QUESTIONS).then(function (answers) {
    // 创建目录
    ['components', 'constant', 'images', 'pages', 'third'].forEach(function (dir) {
      console.log('\u521B\u5EFA\u9879\u76EE\u76EE\u5F55'.green, dir.blue);
      (0, _fs.existsSync)(dir) || (0, _fs.mkdirSync)(dir);
    });

    // 创建文件
    var excludes = ['app-sub.js', 'app-main.js'];
    excludes = handleExcludes(excludes, answers);

    var boilerplateDir = _path2.default.resolve(__dirname, '../boilerplate/app');
    (0, _scaffold.createAppFiles)(answers, boilerplateDir, null, excludes);

    // 创建模块
    var componentName = 'component-sample';
    (0, _scaffold.createModuleFiles)(componentName, 2);
    var pageName = 'index';
    (0, _scaffold.createModuleFiles)(pageName, 1);

    console.log('\u6CE8\u518C\u7EC4\u4EF6'.green, componentName.blue, '\u5230\u9875\u9762'.green, pageName.blue);
    var indexJsonPath = 'pages/index/index.json';
    var indexJson = _fsExtra2.default.readJsonSync(indexJsonPath);
    indexJson.usingComponents['component-sample'] = '/components/component-sample/component-sample';
    _fs2.default.writeFileSync(indexJsonPath, JSON.stringify(indexJson, null, _constant.INDENT));

    setIndexPage();
    (0, _common.executeInit)();
    (0, _update2.default)();
    (0, _lifecycle.afterCommand)({
      compareVersion: false
    });
  });
};

// 添加新的移除项


function handleExcludes(excludes, options) {
  var newExcludes = excludes;
  if (options['digitalgd/index'] === '否') {
    newExcludes.push('digitalgd/index');
  }
  return newExcludes;
}

//设置初始首页
function setIndexPage() {
  var jsonConfig = _fsExtra2.default.readJsonSync('app.json');

  var index = jsonConfig['pages'].indexOf(_constant.DIGITALGD_INDEX);
  if (~index || index === 0) {
    return false;
  }
  var indexPage = jsonConfig['pages'].splice(index, 1);
  jsonConfig['pages'].unshift(indexPage[0]);
  _fs2.default.writeFileSync('app.json', JSON.stringify(jsonConfig, null, _constant.INDENT));
}