'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fsReaddirRecursive = require('fs-readdir-recursive');

var _fsReaddirRecursive2 = _interopRequireDefault(_fsReaddirRecursive);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _md = require('md5');

var _md2 = _interopRequireDefault(_md);

var _constant = require('./constant');

var _core = require('./core');

var _jszip = require('jszip');

var _jszip2 = _interopRequireDefault(_jszip);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var zip = new _jszip2.default();

exports.default = function (src, dest) {
  var CDN_CONFIG = (0, _core.getLocalConfig)().CDN || {};
  var _CDN_CONFIG$IGNORE_ST = CDN_CONFIG.IGNORE_STATIC_FOLDERS,
      IGNORE_STATIC_FOLDERS = _CDN_CONFIG$IGNORE_ST === undefined ? [] : _CDN_CONFIG$IGNORE_ST;


  return new Promise(function (resolve, reject) {
    (0, _core.logStepCaption)('CDN 静态资源替换');

    var _getWorkDirs = (0, _core.getWorkDirs)(),
        DIST_DIR = _getWorkDirs.DIST_DIR,
        WESHOP_WORK_DIR = _getWorkDirs.WESHOP_WORK_DIR;

    src = src || DIST_DIR;
    dest = dest || DIST_DIR;

    // 初始化 ZIP 包
    var zipDirPath = _initZIP();

    // 遍历文件
    var source = (0, _fsReaddirRecursive2.default)(src);
    var deleteDir = new Set();
    source.forEach(function (pth) {
      if ((0, _core.multiMinmatch)(pth, ['**/*.js', '**/*.wxml'])) {
        // console.log(`替换 CDN: ${path.relative(DIST_DIR, path.join(src, pth))}`.grey);
        var content = _fsExtra2.default.readFileSync(_path2.default.join(src, pth), 'utf8').toString();
        var matches = content.match(_constant.PATTERNS.IMAGES);
        matches && matches.forEach(function (originPath) {
          var filePath = _path2.default.join(src, pth);
          // 去除两端括号
          originPath = originPath.replace(/(^[\'|\"]|[\'\"]$)/g, '');
          console.log(('\n\u8D44\u6E90\u5730\u5740\uFF1A' + originPath).grey);

          // 合法资源地址
          if (originPath[0] === '.' || originPath[0] === '/') {
            var imagePath = originPath[0] === '.' ? _path2.default.join(filePath, '../', originPath) // 相对路径
            : _path2.default.join(src, originPath); // 绝对路径

            // 忽略某些特殊标记的文件夹
            if (IGNORE_STATIC_FOLDERS.some(function (ignorePath) {
              return imagePath.includes(ignorePath);
            })) {
              console.log('\u7B26\u5408\u5FFD\u7565\u89C4\u5219\uFF0C\u5DF2\u5FFD\u7565\u8BE5\u6587\u4EF6'.grey);
              return false;
            }

            // 通过资源地址找到相应资源文件 
            if (!_fsExtra2.default.existsSync(imagePath)) {
              console.log(('\n                \u627E\u4E0D\u5230\u8BE5\u9759\u6001\u8D44\u6E90\n                \u9759\u6001\u8D44\u6E90\uFF1A' + originPath + '\n                \u7D22\u5F15\u5730\u5740\uFF1A' + imagePath + '\n                \u6587\u4EF6\u5730\u5740\uFF1A' + filePath + '\n              ').red);
              return false;
            }

            // 检查文件大小
            _checkFileSize(imagePath);

            // 替换资源
            var filename = _getFileMD5Name(imagePath);
            _imagePathReplace(filePath, originPath, filename);

            // 写入 ZIP 包
            var fileContent = _fsExtra2.default.readFileSync(imagePath);
            zip.file(_path2.default.join(zipDirPath, filename), fileContent);

            // 将静态资源文件夹加入到可删除文件夹
            var dir = _path2.default.parse(imagePath).dir;
            deleteDir.add(dir);
          } else if (_constant.PATTERNS.URL.test(originPath)) {
            // 远端地址
            // console.log(`
            //   找到特殊的远端资源
            //   远端资源：${originPath}
            //   文件地址：${filePath}
            // `.yellow)
          } else if (/^\w+/.test(originPath)) {
            // 可能是特殊的资源地址
            console.log(('\n              \u627E\u5230\u7279\u6B8A\u7684\u9759\u6001\u8D44\u6E90\uFF0C\u9700\u8981\u68C0\u67E5\u5220\u9664\u6587\u4EF6\u5939\u662F\u5426\u53D7\u5F71\u54CD\n              \u9759\u6001\u8D44\u6E90\uFF1A' + originPath + '\n              \u6587\u4EF6\u5730\u5740\uFF1A' + filePath + '\n            ').yellow);
          } else {
            // 不符规范的资源地址
            console.log(('\n              \u627E\u5230\u4E0D\u89C4\u8303\u7684\u9759\u6001\u8D44\u6E90\n              \u9759\u6001\u8D44\u6E90\uFF1A' + originPath + '\n              \u6587\u4EF6\u5730\u5740\uFF1A' + filePath + '\n            ').red);
          }
        });
      }
    });

    // 打包 ZIP 包
    zip.generateNodeStream({
      type: 'nodebuffer',
      streamFiles: true
    }).pipe(_fsExtra2.default.createWriteStream(_path2.default.join(WESHOP_WORK_DIR, 'static.zip'))).on('finish', function () {
      if (deleteDir.size > 0) {
        console.log('\n\u5F00\u59CB\u5220\u9664\u6587\u4EF6\u5939'.yellow);
        Array.from(deleteDir).forEach(function (dir) {
          console.log(('\u68C0\u67E5\u6587\u4EF6\u5939\u5E76\u5F00\u59CB\u5220\u9664\u6587\u4EF6\uFF1A' + dir).yellow);
          // 不能直接删除文件夹，仅仅是删除文件夹内部文件
          _fsExtra2.default.readdirSync(dir).forEach(function (file) {
            var extname = _path2.default.extname(file);
            if (_constant.EXTNAME_IMAGE.includes(extname)) {
              _fsExtra2.default.removeSync(_path2.default.join(dir, file));
            }
          });
        });
      }

      resolve(true);
      console.log('CDN \u8D44\u6E90\u66FF\u6362\u5B8C\u6210'.green);
    });
  });
};

/**
 * 根据 CDN 地址的配置获取 ZIP 文件夹结构
 * 具体操作是掐头去尾
 */


function _initZIP() {
  var CDN_CONFIG = (0, _core.getLocalConfig)().CDN || {};
  var _CDN_CONFIG$CDN_PUBLI = CDN_CONFIG.CDN_PUBLIC_PATH,
      CDN_PUBLIC_PATH = _CDN_CONFIG$CDN_PUBLI === undefined ? _constant.DEFAULT_CDN_PATH : _CDN_CONFIG$CDN_PUBLI;


  var dirArr = [];
  _url2.default.parse(CDN_PUBLIC_PATH).pathname.split('/').forEach(function (dir, index, arr) {
    if (index === 0 || index === arr.length - 1) return false;
    dirArr.push(dir);
  });

  return dirArr.join('/');
}

/**
 * 通过文件获取 MD5 后的替换名
 */
function _getFileMD5Name(filePath) {
  var extname = _path2.default.extname(filePath);
  var md5File = (0, _md2.default)(_fsExtra2.default.readFileSync(filePath));

  return md5File + extname;
}

/**
 * 替换文件内的资源地址，替换为 CDN 地址
 * @param {文件地址} filePath 
 * @param {图片地址} originPath 
 * @param {图片名字} filename 
 */
function _imagePathReplace(filePath, originPath, filename) {
  var CDN_CONFIG = (0, _core.getLocalConfig)().CDN || {};
  var _CDN_CONFIG$CDN_PUBLI2 = CDN_CONFIG.CDN_PUBLIC_PATH,
      CDN_PUBLIC_PATH = _CDN_CONFIG$CDN_PUBLI2 === undefined ? _constant.DEFAULT_CDN_PATH : _CDN_CONFIG$CDN_PUBLI2;


  var remotePath = CDN_PUBLIC_PATH + filename;
  var content = _fsExtra2.default.readFileSync(filePath, 'utf8');
  var newContent = content.replace(originPath, remotePath);
  _fsExtra2.default.writeFileSync(filePath, newContent);
  console.log(('CDN \u5730\u5740\uFF1A' + remotePath).grey);
}

/**
 * 检查文件大小，超出限额的给予提示
 * @param {String} filePath 文件地址
 */
function _checkFileSize(filePath) {
  var CDN_CONFIG = (0, _core.getLocalConfig)().CDN || {};
  var _CDN_CONFIG$IMAGE_MAX = CDN_CONFIG.IMAGE_MAX_SIZE,
      IMAGE_MAX_SIZE = _CDN_CONFIG$IMAGE_MAX === undefined ? _constant.DEFAULT_IMAGE_MAX_SIZE : _CDN_CONFIG$IMAGE_MAX;


  var fileInfo = _fsExtra2.default.statSync(filePath);
  var fileSize = (fileInfo.size / 1024).toFixed(2);

  if (fileSize > IMAGE_MAX_SIZE) {
    console.log(('\n      \u6587\u4EF6 SIZE \u8FC7\u5927\uFF0C\u8BF7\u5148\u63D0\u524D\u538B\u7F29\n      \u8D44\u6E90\u5730\u5740\uFF1A' + filePath + '\n      \u6587\u4EF6\u5927\u5C0F\uFF1A' + fileSize + ' kb\n    ').red);
  } else {
    console.log(('\u6587\u4EF6\u5C3A\u5BF8\uFF1A' + fileSize + ' kb').grey);
  }
}