'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.combineAppJs = combineAppJs;
exports.combineAppJson = combineAppJson;
exports.transferSubAppWindowConfigToPage = transferSubAppWindowConfigToPage;
exports.importSubAppWxssToPage = importSubAppWxssToPage;
exports.replaceModulePath = replaceModulePath;
exports.replaceDependencePath = replaceDependencePath;
exports.replaceAbsolutePath = replaceAbsolutePath;
exports.removeUselessFiles = removeUselessFiles;
exports.removeCommonFiles = removeCommonFiles;
exports.removeAppFiles = removeAppFiles;
exports.removeFiles = removeFiles;
exports.setEnv = setEnv;
exports.combineMainPageJs = combineMainPageJs;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _core = require('./core');

var _tosource = require('tosource');

var _tosource2 = _interopRequireDefault(_tosource);

var _constant = require('./constant');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _fsReaddirRecursive = require('fs-readdir-recursive');

var _fsReaddirRecursive2 = _interopRequireDefault(_fsReaddirRecursive);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var INJECT_METHODS = ['onLaunch', 'onShow', 'onHide'];

var LogCaptions = {};

function combineAppJs(subPackageName) {
  (0, _core.logStepCaption)('合并sub-app.js&&main-app.js文件');

  var _getWorkDirs = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs.DIST_DIR;

  var subPackageDir = _path2.default.resolve(DIST_DIR, subPackageName);
  var subAppJsPath = _path2.default.join(subPackageDir, 'sub-app.js');
  var mainAppJsPath = _path2.default.resolve(DIST_DIR, 'main-app.js');

  if (!_fs2.default.existsSync(subAppJsPath)) {
    throw new ReferenceError('找不到文件'.red + ' ' + _path2.default.relative(DIST_DIR, subAppJsPath).blue + ' ' + '请确认是否拆分app.js文件'.red);
  }

  var mainAppJs = _fs2.default.readFileSync(mainAppJsPath).toString();
  var subAppJs = _fs2.default.readFileSync(subAppJsPath).toString();
  var mainAppOptions = (0, _core.getAppOptions)(mainAppJsPath);
  var subAppOptions = (0, _core.getAppOptions)(subAppJsPath);
  var definedFn = [];

  // 合并globalData
  Object.assign(mainAppOptions.globalData, subAppOptions.globalData || {});
  mainAppJs = (0, _core.wrapAppJsOptions)(mainAppJs, mainAppOptions);
  _fs2.default.writeFileSync(mainAppJsPath, mainAppJs);

  Object.keys(subAppOptions).forEach(function (methodName) {
    if (INJECT_METHODS.indexOf(methodName) > -1) {
      definedFn.push(methodName);
      subAppJs = (0, _core.unShiftCode)('const ' + methodName + '=' + (0, _tosource2.default)(subAppOptions[methodName]) + ';', subAppJs);
      subAppJs = (0, _core.unShiftCode)('let ' + methodName + 'Executed=false;', subAppJs);
      subAppOptions[methodName] = new Function(['if(' + methodName + 'Executed === true) return', methodName + 'Executed = true', methodName + '.apply(this, arguments);'].join(';'));
    }
  });

  subAppJs = (0, _core.wrapAppJsOptions)(subAppJs, subAppOptions);
  _fs2.default.writeFileSync(subAppJsPath, subAppJs);

  var localConfig = (0, _core.getLocalConfig)();
  var source = (0, _fsReaddirRecursive2.default)(subPackageDir);
  var fAll = ['pages/**/*.js'].map(function (pth) {
    return _path2.default.join(DIST_DIR, subPackageName, pth);
  });

  source.forEach(function (pth) {
    pth = _path2.default.join(subPackageDir, pth);
    if ((0, _core.multiMinmatch)(pth, fAll)) {
      console.log(('\u626B\u63CF\u6587\u4EF6\uFF1A' + _path2.default.join(subPackageName, pth)).grey);
      var content = _fs2.default.readFileSync(pth).toString();
      if (content.match(_constant.PATTERNS.PAGE_CONSTRUCTOR)) {
        if (!LogCaptions[_path2.default.relative(DIST_DIR, pth)]) {
          console.log(('\u5904\u7406\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, pth)).green);
          LogCaptions[_path2.default.relative(DIST_DIR, pth)] = true;
        }
        var options = (0, _core.getPageOptions)(pth);
        var fileDir = _path2.default.dirname(pth);
        var relToSubAppJs = _path2.default.relative(fileDir, subAppJsPath);

        // 注入sub-app.js生命周期
        INJECT_METHODS.forEach(function (appMethodName) {
          var pageMethodName = appMethodName === 'onLaunch' ? 'onLoad' : appMethodName;
          var pageMethod = options[pageMethodName];
          // 注入埋点代码
          var _fun = [];
          var isRewrite = false;

          if (localConfig.report && localConfig.report[pageMethodName]) {
            _fun.push(localConfig.report[pageMethodName]);
            isRewrite = true;
          }

          if (definedFn.indexOf(appMethodName) > -1) {
            _fun.push('subApp.' + appMethodName + '.apply(getApp(), arguments)');
            isRewrite = true;
          }

          if (isRewrite) {
            if (pageMethod) {
              content = (0, _core.unShiftCode)('const ' + pageMethodName + '=' + (0, _tosource2.default)(pageMethod) + ';', content);
              _fun.push(pageMethodName + '.apply(this, arguments)');
            }
            options[pageMethodName] = new Function(_fun.join(';'));
          }

          // if (definedFn.indexOf(appMethodName) > -1) {
          //   if (pageMethod) {
          //     _fun = [
          //       reportCode,
          //       `subApp.${appMethodName}.apply(getApp(), arguments)`,
          //       `${pageMethodName}.apply(this, arguments)`,
          //     ].join(';')
          //     console.log('  页面生命周期函数', pageMethodName.blue, '已定义', '注入', appMethodName.blue);
          //   } else {
          //      _fun = [
          //       reportCode,
          //       `subApp.${appMethodName}.apply(getApp(), arguments)`,
          //     ].join(';')
          //     console.log('  页面生命周期函数', pageMethodName.blue, '未定义', '定义并注入', appMethodName.blue);
          //   }
          // } else if( reportConfig[pageMethodName] ){ 
          //   if (pageMethod) {
          //     content = unShiftCode(`const ${pageMethodName}=${toSource(pageMethod)};`, content);
          //     _fun = [
          //       reportCode,
          //       `${pageMethodName}.apply(this, arguments)`,
          //     ].join(';')
          //     console.log('  页面生命周期函数', pageMethodName.blue, '已定义', '注入', appMethodName.blue);
          //   } else {
          //      _fun = [
          //       reportCode,
          //     ].join(';')
          //     console.log('  页面生命周期函数', pageMethodName.blue, '未定义', '定义并注入', appMethodName.blue);
          //   }
          // }
        });

        console.log('  注入sub-app.js引用');
        // 注入sub-app.js引用
        content = (0, _core.unShiftCode)('const subApp=require(\'' + relToSubAppJs.replace(/\\/g, '\/') + '\');', content);

        content = (0, _core.wrapPageOptions)(content, options);
        _fs2.default.writeFileSync(pth, content);
      }
    }
  });
  LogCaptions = {};
}

function combineAppJson(subPackageName) {
  (0, _core.logStepCaption)('合并app.json文件');
  var PROJECT_ROOT = (0, _core.findProjectRoot)();

  var _getWorkDirs2 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs2.DIST_DIR;

  var appJsonPath = _path2.default.join(DIST_DIR, 'app.json');
  var subAppJsonPath = _path2.default.join(DIST_DIR, subPackageName, 'app.json');
  var appJson = _fsExtra2.default.readJsonSync(appJsonPath);
  var subAppJson = _fsExtra2.default.readJsonSync(subAppJsonPath);
  var localDepPaths = (0, _core.getLocalDepPaths)(function (p) {
    return _path2.default.relative(PROJECT_ROOT, p);
  });

  var localRemovePaths = (0, _core.getLocalRemovePaths)(function (p) {
    return _path2.default.relative(PROJECT_ROOT, p);
  });

  appJson.subPackages = appJson.subPackages || [];
  // 替换pages中对dgd-weui components的引用
  var filteredPages = [];
  subAppJson.pages.forEach(function (page) {

    var isCommonPage = false;
    var isUseless = false;

    //如果是无用模块
    localRemovePaths.forEach(function (lrp) {
      lrp = lrp.replace(/\\/g, '\/');
      if (page.indexOf(lrp) > -1) {
        isUseless = true;
      }
    });

    // 如果是无用模块，不执行下面步骤
    if (isUseless) {
      return;
    }

    localDepPaths.forEach(function (ldp) {
      ldp = ldp.replace(/\\/g, '\/');
      // 页面是公共模块，        // 且不在合包后的app.json#pages中
      if (page.indexOf(ldp) > -1) {
        isCommonPage = true;
        if (appJson.pages.indexOf(page) < 0) {
          appJson.pages.push(page);
        }
      }
    });
    if (!isCommonPage) {
      filteredPages.push(page);
      // 转移subApp的app.json中window配置到各页面的json中
      transferSubAppWindowConfigToPage(_path2.default.join(DIST_DIR, subPackageName, page + '.json'), subAppJson);
      // 子模块页面引用app.wxss;
      importSubAppWxssToPage(_path2.default.join(DIST_DIR, subPackageName, page + '.wxss'), subPackageName);
    }
  });
  appJson.subPackages.push({
    root: subPackageName,
    pages: filteredPages
  });
  _fs2.default.writeFileSync([DIST_DIR, 'app.json'].join(_constant.SEP), JSON.stringify(appJson, null, _constant.INDENT));
}

function transferSubAppWindowConfigToPage(pageConfigPath, subAppJson) {
  if (!subAppJson.window) {
    return false;
  }

  var _getWorkDirs3 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs3.DIST_DIR;

  var pageConfig = _fsExtra2.default.readJsonSync(pageConfigPath);
  console.log(('\u5904\u7406\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, pageConfigPath)).green);
  Object.keys(subAppJson.window).forEach(function (prop) {
    if (pageConfig[prop] === undefined) {
      console.log('  ', '\u8D4B\u503C', ('' + prop).blue, '>>', ('' + subAppJson.window[prop]).blue);
      pageConfig[prop] = subAppJson.window[prop];
    }
  });

  _fs2.default.writeFileSync(pageConfigPath, JSON.stringify(pageConfig, null, _constant.INDENT));
}

function importSubAppWxssToPage(pageWxssPath, subPackageName) {
  var _getWorkDirs4 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs4.DIST_DIR;

  var pageWxss = _fs2.default.readFileSync(pageWxssPath);
  console.log('\u5904\u7406\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, pageWxssPath));
  console.log('  注入app.wxss引用: ', ('@import "/' + subPackageName + '/app.wxss";').blue);
  _fs2.default.writeFileSync(pageWxssPath, ['@import "/' + subPackageName + '/app.wxss";', pageWxss.toString()].join(_constant.EOL));
}

/**
 * 替换模块引用路径
 *
 * 1. 引用模块是公共模块，则修改为相对到主模块的公共模块路径
 * 2. 引用模块非公共模块，但使用的绝对地址，则相对到本模块对应的路径
 *
 * @param refPath
 * @param filePath
 * @param commonPaths
 * @param subPackageName
 */
function replaceModulePath(refPath, filePath, commonPaths, subPackageName) {
  var _getWorkDirs5 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs5.DIST_DIR;

  var fileDir = _path2.default.dirname(filePath);
  var isRefPathAbsolute = refPath.indexOf('/') === 0;
  var modulePath = isRefPathAbsolute ? refPath.slice(1) : _path2.default.relative(_path2.default.join(DIST_DIR, subPackageName), _path2.default.join(fileDir, refPath)).replace(/\\/g, '\/');

  var isCommon = false;
  var destPath = void 0;

  commonPaths.forEach(function (commonPath) {
    commonPath = commonPath.replace(/\\/g, '\/');
    if (modulePath.indexOf(commonPath) === 0) {
      isCommon = true;
      return false;
    }
  });
  destPath = (isCommon ? _path2.default.relative(fileDir, _path2.default.join(DIST_DIR, modulePath)) : isRefPathAbsolute ? _path2.default.relative(fileDir, _path2.default.join(DIST_DIR, subPackageName, modulePath)) : refPath).replace(/\\/g, '\/');
  if (!LogCaptions[_path2.default.relative(DIST_DIR, filePath)]) {
    console.log(('replaceModulePath \u5904\u7406\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, filePath)).green);
    LogCaptions[_path2.default.relative(DIST_DIR, filePath)] = true;
  }
  if (destPath !== refPath) {
    console.log('  ', '替换路径: ', refPath.blue, ' >> ', destPath.blue);
  }
  return destPath;
}

/**
 * 替换公共依赖路径
 */
function replaceDependencePath(subPackageName) {
  var PROJECT_ROOT = (0, _core.findProjectRoot)();

  var _getWorkDirs6 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs6.DIST_DIR;

  var fJs = ['pages/**/*.js', 'components/**/*.js', 'third/**/*.js', 'sub-app.js'].map(function (pth) {
    return _path2.default.join(DIST_DIR, subPackageName, pth);
  });

  var fJson = ['components/**/*.json', 'pages/**/*.json'].map(function (pth) {
    return _path2.default.join(DIST_DIR, subPackageName, pth);
  });

  var fWxss = ['components/**/*.wxss', 'pages/**/*.wxss', 'third/**/*.wxss', 'app.wxss'].map(function (pth) {
    return _path2.default.join(DIST_DIR, subPackageName, pth);
  });

  var fAll = ['pages/**/*.+(js|json|wxss)', 'components/**/*.+(js|json|wxss)', 'third/**/*.+(js|json|wxss)', 'sub-app.js', 'app.wxss'].map(function (pth) {
    return _path2.default.join(DIST_DIR, subPackageName, pth);
  });

  var localDepPaths = (0, _core.getLocalDepPaths)(function (p) {
    return (0, _path.relative)(PROJECT_ROOT, p);
  });

  var source = (0, _fsReaddirRecursive2.default)(_path2.default.join(DIST_DIR, subPackageName));
  (0, _core.logStepCaption)('替换公共模块引用路径');
  source.forEach(function (pth) {
    console.log(('\u626B\u63CF\u6587\u4EF6\uFF1A' + _path2.default.join(subPackageName, pth)).grey);
    pth = _path2.default.join(DIST_DIR, subPackageName, pth);
    if ((0, _core.multiMinmatch)(pth, fAll)) {
      var content = _fs2.default.readFileSync(pth).toString();
      if ((0, _core.multiMinmatch)(pth, fJs)) {
        // 替换js中的require引入
        content = content.replace(/require\((['"])([^'"]+)\1\)/g, function (m, n, p) {
          return 'require(\'' + replaceModulePath(p, pth, localDepPaths, subPackageName) + '\')';
        });
      } else if ((0, _core.multiMinmatch)(pth, fJson)) {
        var jsonContent = content && JSON.parse(content);

        jsonContent.usingComponents && Object.keys(jsonContent.usingComponents).forEach(function (name) {
          jsonContent.usingComponents[name] = replaceModulePath(jsonContent.usingComponents[name], pth, localDepPaths, subPackageName);
        });
        content = JSON.stringify(jsonContent, null, '  ');
      } else if ((0, _core.multiMinmatch)(pth, fWxss)) {
        content = content.replace(/\@import\s+(['"])([^'"]+)\1/g, function (m, n, p) {
          return '@import "' + replaceModulePath(p, pth, localDepPaths, subPackageName) + '"';
        });
      }
      _fs2.default.writeFileSync(pth, content);
    }
  });
  // 用完清空
  LogCaptions = {};
}

function replaceAbsolutePath(subPackageName) {
  var _getWorkDirs7 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs7.DIST_DIR;

  var source = (0, _fsReaddirRecursive2.default)(_path2.default.join(DIST_DIR, subPackageName));
  var fAll = ['pages/**/*.+(js|wxss|wxml)', 'components/**/*.+(js|wxss|wxml)', 'third/**/*.+(js|wxss|wxml)', 'sub-app.js', 'app.wxss'].map(function (pth) {
    return _path2.default.join(DIST_DIR, subPackageName, pth);
  });
  (0, _core.logStepCaption)('替换绝对路径');
  source.forEach(function (pth) {
    console.log(('\u626B\u63CF\u6587\u4EF6\uFF1A' + _path2.default.join(subPackageName, pth)).grey);
    pth = _path2.default.join(DIST_DIR, subPackageName, pth);
    if ((0, _core.multiMinmatch)(pth, fAll)) {
      var content = _fs2.default.readFileSync(pth).toString();
      var mainPackageName = (0, _core.getProjectNameFromGitUrl)((0, _core.getLocalConfig)().mainPackage);
      content = content.replace(/(["'`])(\/pages(?:(?=(\\?))\3.)*?)\1/g, function (m, n, p) {
        if (!LogCaptions[_path2.default.relative(DIST_DIR, pth)]) {
          console.log(('\u5904\u7406\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, pth)).green);
          LogCaptions[_path2.default.relative(DIST_DIR, pth)] = true;
        }
        console.log('  ', '替换路径: ', p.blue, ' >> ', ('/' + subPackageName + p).blue);
        return n + '/' + subPackageName + p + n;
      });
      content = content.replace(new RegExp('(["\'`])(\\/' + mainPackageName + '(?:(?=(\\\\?))\\3.)*?)\\1', 'g'), function (m, n, p) {
        var pathParts = p.slice(1).split('/');
        var packageName = pathParts.shift();
        if (packageName === mainPackageName) {
          if (!LogCaptions[_path2.default.relative(DIST_DIR, pth)]) {
            console.log(('\u5904\u7406\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, pth)).green);
            LogCaptions[_path2.default.relative(DIST_DIR, pth)] = true;
          }
          console.log('  ', '替换路径: ', p.blue, ' >> ', ('/' + subPackageName + p).blue);
          m = n + '/' + pathParts.join('/') + n;
        }
        return m;
      });
      content = content.replace(/([-a-zA-Z0-9@:%_\+.~#?&//=]+)\.(jpg|jpeg|gif|png|bmp|tiff|tga|svg)/g, function (m, p) {
        if (m[0] === '/') {
          m = ['/', subPackageName, p].join('');
          if (!LogCaptions[_path2.default.relative(DIST_DIR, pth)]) {
            console.log(('\u5904\u7406\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, pth)).green);
            LogCaptions[_path2.default.relative(DIST_DIR, pth)] = true;
          }
          console.log(' ', '替换路径: ', p.blue, ' >> ', m.blue);
        }
        return m;
      });

      _fs2.default.writeFileSync(pth, content);
    }
  });
  // 用完清空
  LogCaptions = {};
}

function removeUselessFiles(subPackageName) {
  (0, _core.logStepCaption)('删除无用文件');
  var PROJECT_ROOT = (0, _core.findProjectRoot)();
  removeFiles((0, _core.getLocalRemovePaths)(function (pth) {
    return _path2.default.relative(PROJECT_ROOT, pth);
  }), subPackageName);
}

function removeCommonFiles(subPackageName) {
  (0, _core.logStepCaption)('删除子模块公共模块文件');
  var PROJECT_ROOT = (0, _core.findProjectRoot)();
  removeFiles((0, _core.getLocalDepPaths)(function (pth) {
    return _path2.default.relative(PROJECT_ROOT, pth);
  }), subPackageName);
}

function removeAppFiles(subPackageName) {
  (0, _core.logStepCaption)('删除子模块app.*，*.config.json文件');
  removeFiles(['app.js', 'app.json', 'project.config.json', 'weshop.config.json'], subPackageName);
}

function removeFiles(paths, subPackageName) {
  var _getWorkDirs8 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs8.DIST_DIR;

  paths.forEach(function (pth) {
    pth = _path2.default.join(DIST_DIR, subPackageName, pth);
    console.log('删除文件: '.green, pth.blue);
    _fsExtra2.default.removeSync(pth);
  });
}

/**
 * 
 * @param {String} env 设置环境
 * @param {String} src 项目目录
 * @param {String} name 包名
 */
function setEnv(env, src, name) {
  (0, _core.logStepCaption)('设置环境变量');

  var _getWorkDirs9 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs9.DIST_DIR;

  var WESHOP_CONFIG = (0, _core.getLocalConfig)();
  var PROJECT_CONFIG_PATH = _path2.default.join(DIST_DIR, _constant.PROJECT_CONFIG_FILE_NAME);
  var PROJECT_ENV_PATH = _path2.default.join(DIST_DIR, _constant.ENV_FILE_NAME);
  var mainPackageName = (0, _core.getProjectNameFromGitUrl)(WESHOP_CONFIG.mainPackage);
  var envConfigPath = _path2.default.join(src, _constant.CONFIG_FILE_NAME);
  var envFilePath = _path2.default.join(src, _constant.ENV_FILE_NAME);

  // 处理 project.config.json
  var PROJECT_CONFIG = _fsExtra2.default.readJsonSync(PROJECT_CONFIG_PATH);
  var envConfig = WESHOP_CONFIG.env[env];
  PROJECT_CONFIG.projectname = envConfig.projectName || PROJECT_CONFIG.projectname;
  PROJECT_CONFIG.appid = envConfig.appid || PROJECT_CONFIG.appid;

  // 修改配置直接注入到 config 里
  console.log('修改', _path2.default.relative(DIST_DIR, envConfigPath).blue, '配置');
  _fs2.default.writeFileSync(PROJECT_CONFIG_PATH, JSON.stringify(PROJECT_CONFIG));
  _fs2.default.writeFileSync(envConfigPath, 'module.exports = ' + JSON.stringify(envConfig));
  _fs2.default.writeFileSync(envFilePath, 'module.exports = \'' + env + '\';');
  // 将执行命令的环境变量带出给主包项目中
  console.log(('' + PROJECT_ENV_PATH).red);
  console.log(('\u73AF\u5883\u53D8\u91CF' + env).red);
  _fs2.default.writeFileSync(PROJECT_ENV_PATH, 'module.exports = \'' + env + '\';');
}

/**
 * 处理主包的埋点
 */
function combineMainPageJs() {
  var localConfig = (0, _core.getLocalConfig)();
  if (!localConfig.report) return;
  var reportConfig = localConfig.report;

  var _getWorkDirs10 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs10.DIST_DIR;

  var packageName = _path2.default.resolve(DIST_DIR);
  var source = (0, _fsReaddirRecursive2.default)(packageName);
  var fAll = ['pages/**/*.js'].map(function (pth) {
    return _path2.default.join(DIST_DIR, pth);
  });
  source.forEach(function (pth) {
    pth = _path2.default.join(packageName, pth);
    if ((0, _core.multiMinmatch)(pth, fAll)) {
      console.log(('\u626B\u63CF\u6587\u4EF6\uFF1A' + _path2.default.join(packageName, pth)).grey);
      var content = _fs2.default.readFileSync(pth).toString();
      if (content.match(_constant.PATTERNS.PAGE_CONSTRUCTOR)) {
        if (!LogCaptions[_path2.default.relative(DIST_DIR, pth)]) {
          console.log(('\u5904\u7406\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, pth)).green);
          LogCaptions[_path2.default.relative(DIST_DIR, pth)] = true;
        }
        var options = (0, _core.getPageOptions)(pth);
        var PAGE_INJECT_METHODS = ['onLoad', 'onShow', 'onHide'];
        // 注入代码
        PAGE_INJECT_METHODS.forEach(function (pageMethodName) {
          var pageMethod = options[pageMethodName];
          // 注入埋点代码
          if (!reportConfig[pageMethodName]) return;
          var reportCode = reportConfig[pageMethodName];
          var _fun = [reportCode];
          if (pageMethod) {
            content = (0, _core.unShiftCode)('const ' + pageMethodName + '=' + (0, _tosource2.default)(pageMethod) + ';', content);
            _fun.push(pageMethodName + '.apply(this, arguments)');
            console.log('  页面生命周期函数', pageMethodName.blue, '已定义', '注入', pageMethodName.blue);
          }
          options[pageMethodName] = new Function(_fun.join(';'));
        });

        content = (0, _core.wrapPageOptions)(content, options);
        _fs2.default.writeFileSync(pth, content);
      }
    }
  });
}