'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findUnExistedPages = findUnExistedPages;
exports.findUnStandardFiles = findUnStandardFiles;
exports.findUnStandardPages = findUnStandardPages;
exports.findUnRegisteredPages = findUnRegisteredPages;

var _fsReaddirRecursive = require('fs-readdir-recursive');

var _fsReaddirRecursive2 = _interopRequireDefault(_fsReaddirRecursive);

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _core = require('./core');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _constant = require('./constant');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 找出不存在的页面
 * @param fix
 * @param src
 * @returns {boolean}
 */
function findUnExistedPages(_ref) {
  var fix = _ref.fix,
      src = _ref.src;

  (0, _core.logStepCaption)('\u67E5\u627E\u4E0D\u5B58\u5728\u7684\u9875\u9762' + (fix && '，并自动修复' || ''));
  src = src || process.cwd();
  var appJsonPath = _path2.default.join(src, 'app.json');
  var unExisted = [];
  var existed = [];
  var appJson = _fsExtra2.default.readJsonSync(appJsonPath);
  appJson.pages = appJson.pages || [];

  appJson.pages.forEach(function (page) {
    var pagePath = _path2.default.join(src, page + '.js');
    if (!_fs2.default.existsSync(pagePath)) {
      unExisted.push(page);
      if (fix) {
        console.log('删除页面注册'.green, page.blue);
        _fs2.default.writeFileSync(appJsonPath, JSON.stringify(appJson, null, _constant.INDENT));
      } else {
        if ((0, _core.isCommon)(page + '.js')) {
          console.log((page + ' \u662F\u516C\u5171\u9875\u9762\uFF0C\u9700\u66F4\u65B0\u4F9D\u8D56\u6A21\u5757').yellow);
        } else {
          console.log((page + ' \u4E0D\u5B58\u5728').red);
        }
      }
    } else {
      existed.push(page);
    }
  });
  if (fix) {
    appJson.pages = existed;
    _fs2.default.writeFileSync(appJsonPath, JSON.stringify(appJson, null, _constant.INDENT));
  }

  unExisted.length === 0 && console.log('未发现不存在页面'.green);
  return fix || unExisted.length === 0;
}

/**
 * 检查非项目规范文件
 * @param src
 * @param fix
 * @returns {boolean}
 */
function findUnStandardFiles(_ref2) {
  var src = _ref2.src,
      fix = _ref2.fix;

  (0, _core.logStepCaption)('查找不规范文件');
  var unStandard = [];
  _fs2.default.readdirSync(src).map(function (name) {
    if (_constant.FILES.PROJECT_FILES.indexOf(name) < 0 && _constant.FILES.NON_PROJECT_FILES.indexOf(name) < 0) {
      var fileTypeText = _path2.default.dirname(name) ? '文件' : '目录';
      console.log(('\u627E\u5230\u975E\u89C4\u8303' + fileTypeText + ' ' + name).yellow);
      unStandard.push(name);

      if (fix && _crossSpawn2.default.sync('git', ('ls-files --error-unmatch ' + name).split(' '), { shell: true }).status === 0) {
        console.log(('\u5220\u9664' + fileTypeText + ' ' + name).green);
        _fsExtra2.default.removeSync(_path2.default.join(src, name));
        (0, _core.addGitIgnore)(name);
      }
    }
  });
  unStandard.length === 0 && console.log('未发现不规范文件'.green);
  return fix || unStandard.length === 0;
}

/**
 * 找出不规范的页面文件
 * 1. 无法获取options的文件
 * 2. 内部使用了 http 请求
 * @param src
 * @param fix
 * @returns {Array}
 */
function findUnStandardPages(_ref3) {
  var src = _ref3.src;

  (0, _core.logStepCaption)('查找不规范页面文件');
  src = src || process.cwd();
  var source = (0, _fsReaddirRecursive2.default)(_path2.default.join(src, 'pages')).map(function (pth) {
    return _path2.default.join('pages', pth);
  });
  var report = [];
  source.forEach(function (pth) {
    if ((0, _core.multiMinmatch)(pth, ['pages/**/*.js'])) {
      var absPath = _path2.default.join(src, pth);
      var content = _fs2.default.readFileSync(absPath).toString();

      // 检测 options 是否符合规范
      if (content && content.match(_constant.PATTERNS.PAGE_CONSTRUCTOR)) {
        try {
          (0, _core.getPageOptions)(absPath, src);
        } catch (e) {
          report.push({
            path: pth,
            message: e.message
          });
        }
      }

      // 检测是否写死了 http 请求
      if (content && content.match(/^http:\/\//)) {
        console.log(content.match(/^http:\/\//));
      }
    }
  });
  report.forEach(function (o) {
    console.log(o.path);
    console.log(''.padEnd(o.path.length, '-'));
    console.log(('' + o.message).red);
    console.log(_constant.EOL);
  });
  report.length === 0 ? console.log('未发现不规范页面文件'.green) : console.log(('\u4E0D\u89C4\u8303\u9875\u9762\u6587\u4EF6\u6570\uFF1A' + report.length).red);
  return report;
}

/**
 * 找出未注册的页面
 * @param src
 * @param fix
 * @returns {boolean}
 */
function findUnRegisteredPages(_ref4) {
  var src = _ref4.src,
      fix = _ref4.fix;

  (0, _core.logStepCaption)('\u67E5\u627E\u672A\u6CE8\u518C\u7684\u9875\u9762' + (fix && '，并自动修复' || ''));
  src = src || process.cwd();
  var source = (0, _fsReaddirRecursive2.default)(_path2.default.join(src, 'pages')).map(function (f) {
    return _path2.default.join('pages', f);
  });
  var appJsonPath = _path2.default.join(src, 'app.json');
  var appJson = _fsExtra2.default.readJsonSync(appJsonPath);
  var unRegistered = [];
  appJson.pages = appJson.pages || [];

  source.forEach(function (pth) {
    if ((0, _core.multiMinmatch)(pth, ['pages/**/*.js'])) {
      var pageName = pth.replace(_path2.default.extname(pth), '').replace(/\\/g, '\/');
      var partials = pageName.split('\/');
      if (partials.pop() === partials.pop() && appJson.pages.indexOf(pageName) < 0) {
        unRegistered.push(pageName);
        if (fix) {
          console.log('注册页面'.green, pageName.blue);
          appJson.pages.push(pageName);
          _fs2.default.writeFileSync(appJsonPath, JSON.stringify(appJson, null, _constant.INDENT));
        } else {
          console.log(pageName.red, '未注册'.red);
        }
      }
    }
  });
  unRegistered.length === 0 && console.log('未发现未注册页面'.green);
  return fix || unRegistered.length === 0;
}

exports.default = function (_ref5) {
  var fix = _ref5.fix,
      src = _ref5.src;

  src = src || process.cwd();
  findUnRegisteredPages({ fix: fix, src: src });
  findUnExistedPages({ fix: fix, src: src });
  findUnStandardFiles({ fix: fix, src: src });
  return findUnStandardPages({ src: src });
};