'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getGlobalConfig = getGlobalConfig;
exports.setGlobalConfig = setGlobalConfig;
exports.createWorkDir = createWorkDir;
exports.fetchPackage = fetchPackage;
exports.copyPackage = copyPackage;
exports.test = test;
exports.openWeChatIDE = openWeChatIDE;
exports.assembleSubPackage = assembleSubPackage;
exports.assembleMainPackage = assembleMainPackage;
exports.executeAssemble = executeAssemble;
exports.executeInit = executeInit;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _uglify = require('./uglify');

var _uglify2 = _interopRequireDefault(_uglify);

var _cdn = require('./cdn');

var _cdn2 = _interopRequireDefault(_cdn);

var _constant = require('./constant');

var _core = require('./core');

var _combine = require('./combine');

var _doctor = require('./doctor');

var _doctor2 = _interopRequireDefault(_doctor);

var _lint = require('./lint');

var _lint2 = _interopRequireDefault(_lint);

var _update = require('./update');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getGlobalConfig() {
  var _getWorkDirs = (0, _core.getWorkDirs)(),
      GLOBAL_CONFIG_PATH = _getWorkDirs.GLOBAL_CONFIG_PATH;

  var PROJECT_ROOT = (0, _core.findProjectRoot)();
  var localConfigPath = _path2.default.resolve(PROJECT_ROOT, GLOBAL_CONFIG_PATH);

  return _fs2.default.existsSync(_path2.default.resolve(PROJECT_ROOT, GLOBAL_CONFIG_PATH)) ? _fsExtra2.default.readJsonSync(localConfigPath) : {};
}

function setGlobalConfig(config) {
  var PROJECT_ROOT = (0, _core.findProjectRoot)();

  var _getWorkDirs2 = (0, _core.getWorkDirs)(),
      GLOBAL_CONFIG_PATH = _getWorkDirs2.GLOBAL_CONFIG_PATH;

  var localConfig = getGlobalConfig(),
      localConfigPath = _path2.default.resolve(PROJECT_ROOT, GLOBAL_CONFIG_PATH);

  Object.assign(localConfig, config);

  _fs2.default.writeFileSync(localConfigPath, JSON.stringify(localConfig, null, _constant.INDENT));
  return localConfig;
}

/**
 * 创建工作目录
 */
function createWorkDir() {
  var _getWorkDirs3 = (0, _core.getWorkDirs)(),
      WESHOP_WORK_DIR = _getWorkDirs3.WESHOP_WORK_DIR,
      WESHOP_DIRS = _getWorkDirs3.WESHOP_DIRS;

  var weshoprc = _fs2.default.existsSync('.weshoprc') && _fsExtra2.default.readJsonSync('.weshoprc') || {};
  // 删除旧版工作目录
  _fs2.default.existsSync('.weshop') && _fsExtra2.default.removeSync('.weshop');
  // 创建工作目录
  _fs2.default.existsSync(weshoprc.dist) && _fsExtra2.default.removeSync(weshoprc.dist);
  _fs2.default.existsSync(WESHOP_WORK_DIR) && _fsExtra2.default.removeSync(WESHOP_WORK_DIR);
  _fsExtra2.default.ensureDirSync(WESHOP_WORK_DIR);

  WESHOP_DIRS.forEach(function (dir) {
    console.log(('\u521B\u5EFAWeShop\u5DE5\u4F5C\u76EE\u5F55\uFF1A' + dir).green);
    _fs2.default.existsSync(dir) || _fs2.default.mkdirSync(dir);
  });

  _fs2.default.writeFileSync('.weshoprc', JSON.stringify({
    WORK_DIR: WESHOP_WORK_DIR
  }, null, _constant.INDENT));
  (0, _core.addGitIgnore)('.weshoprc');
}

/**
 * 获取包代码
 * @param gitUrl
 * @returns {boolean}
 */
function fetchPackage(gitUrl) {
  // 获取/更新主项目 git pull
  var PROJECT_ROOT = (0, _core.findProjectRoot)();

  var _getWorkDirs4 = (0, _core.getWorkDirs)(),
      PACKAGES_DIR = _getWorkDirs4.PACKAGES_DIR;

  var _parseGitUrl = (0, _core.parseGitUrl)(gitUrl),
      repository = _parseGitUrl.repository,
      branch = _parseGitUrl.branch,
      packageName = _parseGitUrl.packageName;

  process.chdir(PACKAGES_DIR);

  (0, _core.logStepCaption)('\u83B7\u53D6\u9879\u76EE ' + packageName.blue);

  console.log(('\u83B7\u53D6\u4ED3\u5E93\u5730\u5740\uFF1A' + repository).green);
  console.log(('\u83B7\u53D6\u4ED3\u5E93\u5206\u652F\uFF1A' + branch).green);

  // 检查/删除子项目目录
  if (_fs2.default.existsSync(_path2.default.join(packageName, '.git'))) {
    process.chdir(packageName);
    (0, _core.gitChildProcessParser)('checkout -- .'.split(' '), { shell: true });
  } else {
    // 已存在文件夹则删除;
    _fsExtra2.default.removeSync(packageName);
    (0, _core.gitChildProcessParser)(['clone', repository, packageName], { shell: true });
    process.chdir(packageName);
  }
  (0, _core.gitChildProcessParser)(['fetch'], { shell: true });
  (0, _core.gitChildProcessParser)(['checkout', branch], { shell: true });
  (0, _core.gitChildProcessParser)(['pull'], { shell: true });
  process.chdir(PROJECT_ROOT);
  return true;
}

/**
 * 复制包代码
 * @param src
 * @param dest
 */
function copyPackage(src, dest) {
  (0, _core.logStepCaption)('复制源代码到编译目录');
  var resolve = _path2.default.resolve;

  _constant.FILES.PROJECT_FILES.forEach(function (filePath) {
    console.log('\u590D\u5236', ('' + src + _constant.SEP + filePath).blue, '>>', ('' + dest + _constant.SEP + filePath).blue);
    _fs2.default.existsSync(resolve(src, filePath)) && _fsExtra2.default.copySync(resolve(src, filePath), resolve(dest, filePath));
  });
}

function test(root) {
  root = root ? _path2.default.resolve(root) : process.cwd();

  var cli = getGlobalConfig().wxCliPath;
  if (cli) {
    _crossSpawn2.default.sync(getGlobalConfig().wxCliPath, ['-t', root], { stdio: 'inherit' });
  } else {
    console.log('尚未配置微信开发者工具命令行路径，请使用客户端操作'.red);
  }
}

/**
 * 打开微信开发工具
 */
function openWeChatIDE() {
  console.log('打开微信开发工具...'.green);
  var PROJECT_ROOT = (0, _core.findProjectRoot)();

  var _getWorkDirs5 = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs5.DIST_DIR;

  var wxCliPath = getGlobalConfig().wxCliPath;
  if (wxCliPath) {
    var openWxDev = _crossSpawn2.default.sync(wxCliPath, ['-o', _path2.default.resolve(PROJECT_ROOT, DIST_DIR)], { shell: true, stdio: 'inherit' });
    if (openWxDev.stderr) {
      console.log('打开微信开发工具失败，可删除相关项目再试'.yellow);
    }
  } else {
    console.log('未设置微信开发者工具安装路径'.yellow);
    console.log('项目路径：'.blue);
    console.log(_path2.default.resolve(DIST_DIR).blue);
  }
}

/**
 * 合并子项目
 * @param subPackageName
 * @returns {boolean}
 */
function assembleSubPackage(subPackageName) {
  (0, _combine.removeUselessFiles)(subPackageName);
  (0, _combine.removeCommonFiles)(subPackageName);
  (0, _combine.replaceDependencePath)(subPackageName);
  (0, _combine.replaceAbsolutePath)(subPackageName);
  (0, _combine.combineAppJson)(subPackageName);
  (0, _combine.combineAppJs)(subPackageName);
  (0, _combine.removeAppFiles)(subPackageName);
  console.log('子项目处理结束'.green);
}

function assembleMainPackage(packageName) {
  (0, _combine.combineMainPageJs)(packageName);
  console.log('主包处理结束'.green);
}

/**
 * 执行合包
 * @param gitUrls
 * exitCode: 1. 主项目合包失败；2. 公共模块更新失败
 */
function executeAssemble(gitUrls, options) {
  return new Promise(function (resolve, reject) {
    var env = options.env,
        debug = options.debug;

    gitUrls = gitUrls || [];

    var remote = _crossSpawn2.default.sync('git', ['config', '--get', 'remote.origin.url']);
    var selfUrl = remote.stdout.toString().trim();
    var selfPackageName = (0, _core.parseGitUrl)(selfUrl).packageName;
    var selfBranch = (0, _core.getDirBranch)();
    var WESHOP_CONFIG = (0, _core.getLocalConfig)();

    var _getWorkDirs6 = (0, _core.getWorkDirs)(),
        DIST_DIR = _getWorkDirs6.DIST_DIR,
        PACKAGES_DIR = _getWorkDirs6.PACKAGES_DIR;

    var mainPackageName = void 0;
    var remotePackageGitUrls = void 0;
    var packages = [];
    var report = {};

    if ((0, _core.isMainPackage)()) {
      mainPackageName = (0, _core.parseGitUrl)(selfUrl).packageName;
      gitUrls = remotePackageGitUrls = gitUrls.length > 0 ? [WESHOP_CONFIG.mainPackage].concat(gitUrls) : [WESHOP_CONFIG.mainPackage].concat(WESHOP_CONFIG.subPackages);
      packages = gitUrls.map(function (url) {
        var _parseGitUrl2 = (0, _core.parseGitUrl)(url),
            packageName = _parseGitUrl2.packageName;

        return {
          name: packageName,
          url: url,
          src: _path2.default.join(PACKAGES_DIR, packageName),
          dest: mainPackageName === packageName ? DIST_DIR : _path2.default.join(DIST_DIR, packageName)
        };
      });
    } else {
      mainPackageName = (0, _core.parseGitUrl)(WESHOP_CONFIG.mainPackage).packageName;
      remotePackageGitUrls = [WESHOP_CONFIG.mainPackage].concat(gitUrls);
      packages = [{
        name: mainPackageName,
        url: WESHOP_CONFIG.mainPackage,
        src: _path2.default.join(PACKAGES_DIR, mainPackageName),
        dest: DIST_DIR
      }, {
        name: selfPackageName,
        url: selfUrl,
        branch: selfBranch,
        src: '.',
        dest: _path2.default.join(DIST_DIR, (0, _core.parseGitUrl)(remote.stdout.toString()).packageName)
      }].concat(gitUrls.map(function (url) {
        var packageName = (0, _core.parseGitUrl)(url).packageName;
        return {
          name: packageName,
          url: url,
          src: _path2.default.join(PACKAGES_DIR, packageName),
          dest: _path2.default.join(DIST_DIR, packageName)
        };
      }));
    }
    // 构造报告对象
    packages.forEach(function (_ref) {
      var name = _ref.name,
          url = _ref.url,
          branch = _ref.branch;

      report[name] = {
        isFail: false,
        url: url,
        reasons: [],
        commit: '',
        author: ''
      };

      if (branch) {
        report[name].branch = branch;
      }
    });
    report[selfPackageName].commit = (0, _core.getGitLastCommit)('.');
    report[selfPackageName].author = (0, _core.getGitLastAuthor)('.');

    //  拉取其他远程分支
    remotePackageGitUrls.forEach(function (url) {
      var src = _path2.default.join(PACKAGES_DIR, (0, _core.parseGitUrl)(url).packageName);

      var _parseGitUrl3 = (0, _core.parseGitUrl)(url),
          packageName = _parseGitUrl3.packageName;

      try {
        fetchPackage(url);
        report[packageName].commit = (0, _core.getGitLastCommit)(src);
        report[packageName].author = (0, _core.getGitLastAuthor)(src);
        report[packageName].branch = (0, _core.getGitCurrentBranch)(src);
      } catch (e) {
        report[packageName].isFail = true;
        report[packageName].reasons.push(e.message.red);
        report[packageName].branch = (0, _core.getGitCurrentBranch)(src);
        if (packageName === mainPackageName) {
          console.log('主包代码获取失败，合包终止！'.red);
          console.log(e.message);
          (0, _core.exportAssembleReport)(report);
          process.exit(1);
        } else {
          console.log(e.message.red);
          process.chdir(PROJECT_ROOT);
          return false;
        }
      }
    });

    packages.forEach(function (_ref2) {
      var name = _ref2.name,
          src = _ref2.src,
          dest = _ref2.dest;

      var proceedReport = report[name];
      if (!proceedReport.isFail) {

        try {
          var doctorReport = (0, _doctor2.default)({ fix: true, src: src });
          if (doctorReport.length > 0 && name !== mainPackageName) {
            report[name].isFail = true;
            report[name].reasons.push(('\n              \u9879\u76EE\u89C4\u8303\u6821\u9A8C\u672A\u901A\u8FC7\n              ' + doctorReport.map(function (r) {
              return r.message;
            }).join('\n') + '\n          ').red);
            return false;
          }
        } catch (e) {
          report[name].isFail = true;
          report[name].reasons.push(e.message.red);
          console.log('规范校验执行错误'.red);
          console.log(e.message.red);
          return false;
        }
        try {
          (0, _lint2.default)({ fix: true, src: src });
          copyPackage(src, dest);
          (0, _combine.setEnv)(env, dest, name);

          return true;
        } catch (e) {
          report[name].isFail = true;
          report[name].reasons.push(e.message.red);
          if (name === mainPackageName) {
            console.log('主包代码校验不通过，合包终止！'.red);
            console.log(e.message);
            (0, _core.exportAssembleReport)(report);
            process.exit(1);
          } else {
            console.log(e.message.red);
            return false;
          }
        }
      }
    });

    // 更新依赖，需要去到远程分支下的主包下更新
    try {
      (0, _update.updateDependencies)(report, DIST_DIR);
      // if(!isMainPackage()) {
      //   console.log('我不是主包，需要 copy'.red)
      //   copyDependencies();
      // }
      (0, _doctor.findUnRegisteredPages)({ fix: true, src: DIST_DIR });
    } catch (e) {
      console.log('\u65E0\u6CD5\u66F4\u65B0\u516C\u5171\u6A21\u5757\uFF0C\u5408\u5305\u7EC8\u6B62\uFF01'.red);
      console.log(e.message && e.message.red || e);
      process.exit(2);
    }

    // assembleSubPackage 合并子项目代码
    packages.forEach(function (_ref3) {
      var name = _ref3.name,
          src = _ref3.src,
          dest = _ref3.dest;

      var proceedReport = report[name];
      console.log(proceedReport);
      if (!proceedReport.isFail) {
        try {
          if (name === mainPackageName) {
            assembleMainPackage(name);
          } else {
            // 删除子项目公共模块代码
            assembleSubPackage(name);
          }
          return true;
        } catch (e) {
          report[name].isFail = true;
          report[name].reasons.push(e.message.red);
          console.log(e.message.red);
          return false;
        }
      }
    });

    // 根据不同环境做文件的后处理，如果是 DEBUG 模式，则直接不做任何处理
    var $p = null;
    if (debug) {
      $p = Promise.resolve(true);
    } else if (env === 'test' || env === 'dev') {
      // 开发环境和测试环境仅压缩代码
      $p = (0, _uglify2.default)(DIST_DIR, DIST_DIR, env);
    } else {
      // 正式环境和预发布环境压缩代码并删除 CDN 图片
      $p = (0, _cdn2.default)(DIST_DIR).then(function () {
        return (0, _uglify2.default)(DIST_DIR, DIST_DIR, env);
      });
    }

    // 执行结束后执行报告
    $p.then(function () {
      resolve(report);
    }).catch(function (e) {
      console.log(e.message.red);
      reject(e);
    });
  });
}

function executeInit() {
  console.log('\u521D\u59CB\u5316\u5F00\u59CB...'.green);
  console.log('\u51C6\u5907\u4E00\u4E9B\u5FC5\u987B\u7684\u76EE\u5F55...'.green);
  createWorkDir();

  // 获取微信开发工具安装路径
  if (_os2.default.type().toLowerCase() === 'darwin') {
    setGlobalConfig({
      wxCliPath: '/Applications/wechatwebdevtools.app/Contents/Resources/app.nw/bin/cli'
    });
  }

  // 更新依赖项目
  (0, _update.updateDependencies)();
  console.log('\u521D\u59CB\u5316\u5B8C\u6210.'.green);
}