'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fsReaddirRecursive = require('fs-readdir-recursive');

var _fsReaddirRecursive2 = _interopRequireDefault(_fsReaddirRecursive);

var _core = require('./core');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _eslintrc = require('../config/.eslintrc');

var _eslintrc2 = _interopRequireDefault(_eslintrc);

var _eslint = require('eslint');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _constant = require('./constant');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function combineReport(src, append) {
  if (!src) {
    src = append;
  } else if (append.errorCount > 0 || append.warningCount > 0) {
    src.results = src.results.concat(append.results);
    'errorCount warningCount fixableErrorCount fixableWarningCount'.split(' ').forEach(function (prop) {
      src[prop] += append[prop];
    });
  }
  return src;
}

exports.default = function (_ref) {
  var fix = _ref.fix,
      src = _ref.src,
      download = _ref.download;

  (0, _core.logStepCaption)('代码质量校验');
  src = src || process.cwd();
  var PROJECT_ROOT = (0, _core.findProjectRoot)();
  var configFileName = '.eslintrc.js';
  var reportFileName = 'eslint-report.html';
  var configFilePath = (0, _path.resolve)(__dirname, '../config', configFileName);
  var source = (0, _fsReaddirRecursive2.default)(src);
  var fAll = ['sub-app.js', 'app.js', 'components/**/*.js', 'pages/**/*.js', 'third/**/*.js', 'constant/**/*.js'].concat((0, _core.isMainPackage)() ? ['main-app.js'] : []);
  var cli = new _eslint.CLIEngine({
    baseConfig: _eslintrc2.default,
    fix: fix
  });
  var excludes = [];
  var filtered = [];
  var report = void 0;

  (0, _core.getLocalDepPaths)(function (localDepPath) {
    excludes.push((0, _path.relative)(PROJECT_ROOT, localDepPath).replace(/\\/g, '\/') + '/**/*.js');
  });

  if (download) {
    console.log('\u4E0B\u8F7D\u6587\u4EF6'.green, configFileName.blue, '到本地'.green);
    _fs2.default.copyFileSync(configFilePath, (0, _path.join)(PROJECT_ROOT, configFileName));
    (0, _core.addGitIgnore)(configFileName);
  }

  source.forEach(function (pth) {
    if ((0, _core.multiMinmatch)(pth, fAll) && !(0, _core.multiMinmatch)(pth, excludes)) {
      filtered.push(_path2.default.join(src, pth));
    }
  });
  try {
    filtered.forEach(function (pth) {
      console.log(('\u6821\u9A8C\u6587\u4EF6\uFF1A' + _path2.default.join(_path2.default.basename(src), _path2.default.relative(src, pth))).grey);
      var append = cli.executeOnFiles([pth]);
      if (fix) {
        _eslint.CLIEngine.outputFixes(append);
      }
      if (append.errorCount !== 0 || append.warningCount !== 0) {
        console.log(cli.getFormatter('codeframe')(append.results));
      }
      report = combineReport(report, append);
    });
  } catch (e) {
    throw new Error('\n      \u4EE3\u7801\u6821\u9A8C\u8FD0\u884C\u9519\u8BEF\n      ' + e.message + '\n    ');
  }

  _fs2.default.writeFileSync(reportFileName, cli.getFormatter('html')(report.results));
  console.log('生成报告文件: '.green, reportFileName.blue);
  (0, _core.addGitIgnore)(reportFileName);

  if (report.errorCount !== 0 || report.warningCount !== 0) {
    throw new TypeError(('\n      \u6821\u9A8C\u9519\u8BEF\u603B\u6570\uFF1A' + report.errorCount + '\n      \u6821\u9A8C\u8B66\u544A\u603B\u6570\uFF1A' + report.warningCount + '\n      \u4EE3\u7801\u6821\u9A8C\u672A\u901A\u8FC7\uFF0C\u8BF7\u5728\u9879\u76EE\u76EE\u5F55\u4E0B\u8FD0\u884Cweshop lint --fix\uFF0C\u5E76\u624B\u52A8\u4FEE\u590D\u5269\u4F59\u4E0D\u89C4\u8303\u7684\u4EE3\u7801').red);
  }
};