'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _constant = require('./constant');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 开启测试
 */
function startRelease() {
  getReleaseVersion().then(function (version) {
    // 执行 git flow release start
    var result = _crossSpawn2.default.sync('git', ['flow', 'release', 'start', _constant.RELEASE_VERSION_PREFIX + '_' + version], { shell: true });

    // 捕获常见错误 
    var stderr = result.stderr.toString().trim();

    if (stderr) {
      if (stderr.includes('There is an existing release branch')) {
        console.log('release start 失败，还存在一条尚未结束的 release 分支，请先运行 weshop release finish 结束已生成的 release 分支'.red, ('' + stderr).red);
      } else if (stderr.includes('Not a gitflow-enabled repo yet')) {
        console.log('该仓库未初始化 gitflow 模式，请先按文档在 source-tree 初始化 gitflow 模式'.red, ('' + stderr).red);
      } else if (stderr.includes('\'flow\' is not a git command')) {
        console.log('还未安装 gitflow'.red, ('' + stderr).red);
        console.log('可以看一下相关指引：', 'https://danielkummer.github.io/git-flow-cheatsheet/index.zh_CN.html'.link);
      } else if (stderr.includes('already exists. Pick another name.')) {
        console.log('已存在该远程分支'.red, ('' + stderr).red);
      } else if (stderr.includes('Switched to a new branch')) {
        // 成功执行 release
        // 获取推送源
        var origin = _crossSpawn2.default.sync('git', ['remote', 'show']).stdout.toString().trim() || 'origin';
        var pushResult = _crossSpawn2.default.sync('git', ['push', '--set-upstream', origin, 'release/' + _constant.RELEASE_VERSION_PREFIX + '_' + version], { stdio: 'inherit' });
        console.log('\n            release \u5206\u652F\u5DF2\u751F\u6210\u5E76\u5207\u6362\u5230\u8BE5\u5206\u652F\uFF0C\u53EF\u4EE5\u4F7F\u7528 weshop a --env test \u8FDB\u884C\u529F\u80FD\u6D4B\u8BD5\u3002\n          '.green);
      } else {
        console.log(('release start \u9519\u8BEF\u8F93\u51FA\uFF1A' + stderr).red);
      }
    }
  });
}

/**
 * 结束测试
 */
function finishRelease() {
  getReleaseVersion().then(function (version) {
    // 执行 git flow release finish 
    var result = _crossSpawn2.default.sync('git', ['flow', 'release', 'finish', _constant.RELEASE_VERSION_PREFIX + '_' + version], { stdio: 'inherit' });
    // 捕获常见错误 
    // const stderr = result.stderr.toString().trim()

    // if(stderr) {
    //   if(stderr.includes(`branch 'master' may be fast-forwarded`)) {
    //     console.log('请先到 master 分支上执行 git pull，更新 master 的代码。'.red)
    //   } else if(stderr.includes('does not exist and is required')) {
    //     console.log('release 分支不存在，请先执行 weshop release start。'.red)
    //     console.log(stderr.red)
    //   }  else if(stderr.includes('Not a gitflow-enabled repo yet')) {
    //     console.log('该仓库未初始化 gitflow 模式，请先按文档在 source-tree 初始化 gitflow 模式'.red)
    //   } else if(stderr.includes(`'flow' is not a git command`)) {
    //     console.log('还未安装 gitflow'.red)
    //     console.log('可以看一下相关指引：https://danielkummer.github.io/git-flow-cheatsheet/index.zh_CN.html')
    //   } else {
    //     console.log(`release finish 错误输出：${stderr}`.red)
    //   }
    // }
  });
}

/**
 * 获取统一的发布 version
 */
function getReleaseVersion() {
  return _axios2.default.get(_constant.RELEASE_VERSION_CGI, {
    responseType: 'json'
  }).then(function (res) {
    var version = res.data.version;
    if (typeof version === 'undefined') {
      console.log('获取统一的版本号失败，无 version 数据'.red);
    }

    console.log('\u6210\u529F\u83B7\u53D6 release \u7248\u672C\u53F7\uFF1A' + version);
    return version;
  });
}

exports.default = function (subcommand) {
  if (subcommand === 'start') {
    return startRelease();
  }

  if (subcommand === 'finish') {
    return finishRelease();
  }
};