'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createModuleFiles = createModuleFiles;
exports.registerPage = registerPage;
exports.createAppFiles = createAppFiles;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _constant = require('./constant');

var _core = require('./core');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 创建模块文件
 * @param name 模块名
 * @param type 模块类型（1 - page, 2 - component）
 */
function createModuleFiles(name, type) {
  var PROJECT_ROOT = (0, _core.findProjectRoot)();
  var typeText = type === 1 ? '页面' : '组件';
  var partials = name.split('\/');
  var moduleTypeMap = [null, 'page', 'component'];
  var moduleType = moduleTypeMap[type];
  var moduleName = partials.pop();
  var moduleDir = _path2.default.join(PROJECT_ROOT, moduleType + 's', name);
  var modulePath = _path2.default.join(moduleDir, moduleName);
  var content = '';

  if (_fs2.default.existsSync(moduleDir)) {
    console.log((typeText + '\u76EE\u5F55').red, _path2.default.relative(PROJECT_ROOT, moduleDir).blue, '已存在'.red);
    return false;
  }

  _fsExtra2.default.ensureDirSync(moduleDir);
  console.log(('\u5F00\u59CB\u521B\u5EFA' + typeText).green, _path2.default.relative(PROJECT_ROOT, modulePath).blue);
  process.chdir(moduleDir);
  ['js', 'json', 'wxml', 'wxss'].forEach(function (fileType) {
    var fileName = [modulePath, fileType].join('.');
    content = _fs2.default.readFileSync(_path2.default.resolve(__dirname, _path2.default.join('../boilerplate/', moduleType, [moduleType, fileType].join('.'))));
    _fs2.default.writeFileSync(fileName, content.toString().replace(/{{\s*.+?\s*}}/ig, moduleName));
    console.log('  \u521B\u5EFA\u6587\u4EF6'.green, _path2.default.relative(PROJECT_ROOT, [modulePath, fileType].join('.')).blue);
  });
  if (type === 1) {
    registerPage(modulePath);
    console.log('  \u6CE8\u518C\u9875\u9762'.green, _path2.default.relative(PROJECT_ROOT, modulePath).blue);
  }
  console.log(('' + typeText).green, _path2.default.relative(PROJECT_ROOT, modulePath).blue, '创建完成'.green);
  process.chdir(PROJECT_ROOT);
}

/**
 * 注册页面
 * @param pagePath
 */
function registerPage(pagePath) {
  var PROJECT_ROOT = (0, _core.findProjectRoot)();
  var appConfig = void 0,
      pageRelPath = void 0,
      appJsonPath = _path2.default.join(PROJECT_ROOT, 'app.json');

  appConfig = _fsExtra2.default.readJsonSync(appJsonPath);

  // Linux、Window 路径兼容
  pageRelPath = _path2.default.relative(PROJECT_ROOT, pagePath).split(_path2.default.sep).join('/');
  appConfig.pages.includes(pageRelPath) || appConfig.pages.push(pageRelPath);
  _fs2.default.writeFileSync(appJsonPath, JSON.stringify(appConfig, null, _constant.INDENT));
}

function createAppFiles(options, srcRoot, destRoot, excludes) {
  destRoot = destRoot || '.';
  var files = _fs2.default.readdirSync(srcRoot);
  files.forEach(function (pth) {
    if (excludes && excludes.indexOf(pth) > -1) {
      return false;
    } else {
      var srcPath = _path2.default.join(srcRoot, pth);
      if (_fs2.default.statSync(srcPath).isDirectory()) {
        createAppFiles(options, _path2.default.join(srcRoot, pth), _path2.default.join(destRoot, pth), excludes);
      } else {
        var destPath = _path2.default.join(destRoot, pth);
        var content = _fs2.default.readFileSync(srcPath);
        _fsExtra2.default.ensureFileSync(destPath);
        if (pth === 'app.js') {
          var appContent = _fs2.default.readFileSync(_path2.default.join(srcRoot, 'app-' + (options.type === '子项目' ? 'sub' : 'main') + '.js'));
          console.log('\u521B\u5EFA\u6587\u4EF6'.green, destPath.blue);
          _fs2.default.writeFileSync(destPath, appContent);
          destPath = _path2.default.join(destRoot, (options.type === '子项目' ? 'sub' : 'main') + '-app.js');
          console.log('\u521B\u5EFA\u6587\u4EF6'.green, destPath.blue);
          _fs2.default.writeFileSync(destPath, content);
        } else {
          console.log('\u521B\u5EFA\u6587\u4EF6'.green, destPath.blue);
          _fs2.default.writeFileSync(destPath, content.toString().replace(/{{\s*(.+?)\s*}}/ig, function (m, n) {
            return options[n] || m;
          }));
        }
      }
    }
  });
}