'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = uglify;

var _core = require('./core');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsReaddirRecursive = require('fs-readdir-recursive');

var _fsReaddirRecursive2 = _interopRequireDefault(_fsReaddirRecursive);

var _uglifyEs = require('uglify-es');

var _uglifyEs2 = _interopRequireDefault(_uglifyEs);

var _htmlMinifier = require('html-minifier');

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _cssnano = require('cssnano');

var _cssnano2 = _interopRequireDefault(_cssnano);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 
 * @param {String} src 需要压缩的目录
 * @param {String} dest 压缩后的输出目录
 * @param {String} env 环境变量  
 */
function uglify(src, dest, env) {
  return new Promise(function (resolve, reject) {
    (0, _core.logStepCaption)('代码压缩');

    var _getWorkDirs = (0, _core.getWorkDirs)(),
        DIST_DIR = _getWorkDirs.DIST_DIR;

    src = src || DIST_DIR;
    dest = dest || DIST_DIR;
    var WESHOP_CONFIG = (0, _core.getLocalConfig)();
    var WESHOP_ENV_CONFIG = WESHOP_CONFIG.env[env];

    var source = (0, _fsReaddirRecursive2.default)(src);
    source.forEach(function (pth) {
      if ((0, _core.multiMinmatch)(pth, ['**/*.js'])) {
        console.log(('\u538B\u7F29\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, _path2.default.join(src, pth))).grey);
        var code = _fs2.default.readFileSync(_path2.default.join(src, pth)).toString();
        var result = _uglifyEs2.default.minify(code, {
          parse: {
            ecma: 8
          },
          compress: {
            drop_console: !!WESHOP_ENV_CONFIG.dropConsole,
            ecma: 6
          },
          output: {
            ecma: 6
          }
        });
        result.error && console.log(result.error);
        result.warnings && console.log(result.warnings);
        _fs2.default.writeFileSync(_path2.default.join(dest, pth), result.code || code);
      } else if ((0, _core.multiMinmatch)(pth, ['**/*.json'])) {
        console.log(('\u538B\u7F29\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, _path2.default.join(src, pth))).grey);
        _fs2.default.writeFileSync(_path2.default.join(dest, pth), JSON.stringify(_fsExtra2.default.readJsonSync(_path2.default.join(src, pth))));
      } else if ((0, _core.multiMinmatch)(pth, ['**/*.wxss'])) {
        console.log(('\u538B\u7F29\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, _path2.default.join(src, pth))).grey);
        var _code = _fs2.default.readFileSync(_path2.default.join(src, pth)).toString();
        (0, _postcss2.default)([(0, _cssnano2.default)({
          preset: ['default', {
            calc: false
          }]
        })]).process(_code, {
          from: _path2.default.join(src, pth),
          to: _path2.default.join(dest, pth)
        }).then(function (result) {
          _fs2.default.writeFileSync(_path2.default.join(dest, pth), result.css);
          resolve(result.css);
        }).catch(function (e) {
          console.log(e);
          console.log(_path2.default.relative(DIST_DIR, _path2.default.join(src, pth)).yellow, '未压缩'.yellow);
          return true;
        });
      } else if ((0, _core.multiMinmatch)(pth, ['**/*.wxml'])) {
        console.log(('\u538B\u7F29\u6587\u4EF6: ' + _path2.default.relative(DIST_DIR, _path2.default.join(src, pth))).grey);
        var _code2 = _fs2.default.readFileSync(_path2.default.join(src, pth)).toString();
        try {
          var _result = (0, _htmlMinifier.minify)(_code2, {
            html5: true,
            removeComments: true,
            collapseWhitespace: true,
            removeRedundantAttributes: true,
            removeEmptyAttributes: true,
            keepClosingSlash: true,
            caseSensitive: true
          });
          _fs2.default.writeFileSync(_path2.default.join(dest, pth), _result);
        } catch (e) {
          console.log(_path2.default.relative(DIST_DIR, _path2.default.join(src, pth)).yellow, '未压缩'.yellow);
          return true;
        }
      }
    });
  });
}