'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateDependencies = updateDependencies;
exports.copyDependencies = copyDependencies;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _fsReaddirRecursive = require('fs-readdir-recursive');

var _fsReaddirRecursive2 = _interopRequireDefault(_fsReaddirRecursive);

var _core = require('./core');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 更新依赖库
 * @param branch 当前依赖库分支
 */
function updateDependencies(report, destRoot) {
  (0, _core.logStepCaption)('更新公共模块');
  var PROJECT_ROOT = (0, _core.findProjectRoot)();
  var rootPath = destRoot || PROJECT_ROOT;
  if (!(0, _core.isMainPackage)()) {
    (0, _core.fetchLatestWeshopConfig)(rootPath);
  }
  var WESHOP_CONFIG = (0, _core.getLocalConfig)();
  var mainPackageName = (0, _core.parseGitUrl)(WESHOP_CONFIG.mainPackage).packageName;

  var dependencies = WESHOP_CONFIG.dependencies;
  Array.isArray(dependencies) && dependencies.forEach(function (dependence) {
    var remote = _crossSpawn2.default.sync('git', ['config', '--get', 'remote.origin.url']);
    // todo 统一Git URL的解析

    var _parseGitUrl = (0, _core.parseGitUrl)(dependence.repo),
        packageName = _parseGitUrl.packageName,
        repository = _parseGitUrl.repository,
        branch = _parseGitUrl.branch;

    var dest = _path2.default.join(rootPath, dependence.path || '.');
    var copyRootDir = dependence.copyRootDir;

    // 判断依赖是否自身
    if (remote.stdout.toString().trim() === repository) {
      return false;
    }

    var sparsePaths = Array.isArray(dependence.remotePaths) && dependence.remotePaths.map(function (remotePath) {
      return _path2.default.relative(rootPath, _path2.default.join(dest, remotePath));
    });

    // 通过临时目录拉取代码，并构造依赖的报告
    (0, _core.sparseCheckout)(dependence.repo, sparsePaths, copyRootDir, dest, function (tmpPath) {
      if (report && packageName !== mainPackageName) {
        report[packageName] = {
          name: packageName,
          repository: repository,
          url: dependence.repo,
          commit: (0, _core.getGitLastCommit)(tmpPath),
          author: (0, _core.getGitLastAuthor)(tmpPath),
          branch: branch,
          isFail: false,
          reasons: []
        };
      }
    });

    (0, _core.addGitIgnore)(sparsePaths, copyRootDir);
  });
}

function copyDependencies() {
  (0, _core.logStepCaption)('复制公共模块文件到编译目录');
  var PROJECT_ROOT = (0, _core.findProjectRoot)();

  var _getWorkDirs = (0, _core.getWorkDirs)(),
      DIST_DIR = _getWorkDirs.DIST_DIR;

  var fAll = (0, _core.getLocalDepPaths)(function (pth) {
    return _path2.default.extname(pth) ? pth : _path2.default.join(pth, '**');
  });

  var source = (0, _fsReaddirRecursive2.default)(PROJECT_ROOT);
  source.forEach(function (pth) {
    pth = _path2.default.join(PROJECT_ROOT, pth);
    if ((0, _core.multiMinmatch)(pth, fAll)) {
      console.log('\u590D\u5236', _path2.default.resolve(PROJECT_ROOT, pth).blue, '>>', _path2.default.join(DIST_DIR, _path2.default.relative(PROJECT_ROOT, pth)).blue);
      _fsExtra2.default.copySync(pth, _path2.default.join(DIST_DIR, _path2.default.relative(PROJECT_ROOT, pth)));
    }
  });
}